<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget, which gets and displays suggestions from the Marketing Automation tracker
 */
class MarkaSuggestionsWidget extends BaseWidget
{
    /**
     * The number of suggestions that should be retrieved.
     * @var int 
     */
    public $suggestionsCount;
    
    /**
     * The array of categories that all returned suggestions must have (they may
     * also have other categories as well).
     * This is optional and if set, it must be a list of strings, 
     * e.g. array('category1', 'category2')
     * @var array 
     */
    public $categories = NULL;
    
    /**
     * An optional array of page attributes, which should be retrieved for each
     * suggested page and passed on to the view in addition to 'name' and 'url'.
     * Note that these attributes are CMS attributes like 'filename' or 
     * 'publishdate', not Marketing Automation attributes. For example:
     * array('folder', 'editdate')
     * @var array 
     */
    public $additionalPageAttributes = NULL;
    
    /**
     * The view that should be used to render the suggestions.
     * Default: "MarkaSuggestionsSimpleLinksList"
     * 
     * This view is passed the parameter suggestedPages, which is an array
     * of the suggested pages infos in the order they were provided by
     * Marketing Automation (sorted by descending view count). Each of these
     * objects is an associative array that contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL. If there are no
     * suggested pages the parameter suggestedPages will be NULL.
     * 
     * @var string 
     */
    public $view = "MarkaSuggestionsSimpleLinksList";
    
    /**
     * The prefix for cache entries (suggested page infos).
     * To make sure that suggested page infos with different page attributes
     * (when different additionalPageAttributes were used) are cached 
     * independently, implode($additionalPageAttributes) . '_' is appended to this prefix.
     * @var string 
     */
    private $cachePrefix = 'Marka_';
    
    /**
     * Gets the suggestions from the Marketing Automation tracker, then gets the
     * required page attributes from the content repository,
     * and finally renders the view.
     * 
     * @return void
     */
    public function run()
    {
        if (empty($this->suggestionsCount)) {
            die('No suggestionsCount specified.');
        }
        if (!empty($this->additionalPageAttributes)) {
            $this->cachePrefix .= implode($this->additionalPageAttributes) . '_';
        }
        $markaModule = Yii::app()->getModule('marketingautomation');
        $url = $markaModule->getMarkaTrackerBaseUrl() . '/v1/suggestions/';
        $url .= urlencode($markaModule->getUUID()) . '/' . strval($this->suggestionsCount);
        if (!empty($this->categories)) {
            $url .= '/' . $this->_assembleCategoriesStr($this->categories);
        }
        $responseJson = @file_get_contents($url);
        $suggestedPages = NULL;
        if ($responseJson !== false) {
            $responseObj = json_decode($responseJson);
            if (!empty($responseObj->suggestions)) {
                $suggestedPages = $this->_getSuggestedPagesInfos($responseObj->suggestions);
                if (empty($suggestedPages)) {
                    $suggestedPages = NULL;
                }
            }
        }
        $this->render($this->view, array('suggestedPages' => $suggestedPages));
    }
    
    /**
     * Converts the specified categories array into a categories string for
     * the get suggestions REST request and returns that string. This string
     * is of the form category1;category2;category3;category4
     * @param array $categories
     * @return string
     */
    private function _assembleCategoriesStr(&$categories)
    {
        $ret = '';
        $first = true;
        foreach ($categories as $category) {
            if (!$first) {
                $ret .= ';';
            } else {
                $first = false;
            }
            $ret .= urlencode($category);
        }
        return $ret;
    }
    
    /**
     * Retrieves the requested attributes from the content repository for
     * the specified Marketing Automation page IDs and returns an array
     * with all page infos found in the repository in the order that the page
     * IDs appear in the markaPageIds array. Each of these
     * objects is an associative array that contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL.
     * 
     * @param array $markaPageIds the array of page IDs returned by Marketing Automation
     * @return array
     */
    private function _getSuggestedPagesInfos(&$markaPageIds)
    {
        $parsedPageIds = self::_parsePageIds($markaPageIds);
        $this->_checkForCachedPageInfos($parsedPageIds);
        $pageIdsGroupedByChannel = self::_groupPageIdsByChannel($parsedPageIds);
        foreach ($pageIdsGroupedByChannel as $channelId => $group) {
            $query = array('filter' => self::_createFilterString($group), 'type' => 'php');
            if ($channelId !== 'master') {
                $query['channel_id'] = $channelId;
            }
            $requiredPageAttributes = $this->_createPageAttributesArray();
            $response = YII::app()->repositoryApi->request($query, $requiredPageAttributes);
            $rawPageInfos = self::_prepareResponse($response);
            if ($rawPageInfos === false) {
                continue;
            }
            $this->_createSuggestedPageInfos($rawPageInfos, $channelId, $parsedPageIds, $requiredPageAttributes);
        }
        return self::_createSuggestedPagesInfosArray($parsedPageIds);
    }
    
    /**
     * Parses the specified array of Marketing Automation page IDs and returns
     * an array of parsed page IDs. Each of these is an object, which contains
     * the following properties: posInSuggestions (the index that the page ID has
     * in the markaPageIds array, this is important for maintaining the order
     * of the suggestions in the end), markaPageId (the Marketing Automation page ID),
     * channelId (the channel ID of the page, if present, or NULL otherwise),
     * and pageId (the page ID used by the CMS, this is already prefixed with '10007.').
     * Each array entry has the key 'c' . channelId . '.' . CMSPageId or
     * only CMSPageId (if it has no channelId).
     * @param array $markaPageIds the array of page IDs returned by Marketing Automation
     * @return array
     */
    private static function _parsePageIds(&$markaPageIds)
    {
        $ret = array();
        $size = count($markaPageIds);
        for ($i = 0; $i < $size; ++$i) {
            $currPageId = $markaPageIds[$i];
            $currPage = new stdClass();
            $currPage->posInSuggestions = $i;
            $currPage->markaPageId = $currPageId;
            if (substr($currPageId, 0, 1) === 'c') {
                $separatorIndex = strpos($currPageId, '.', 2);
                if ($separatorIndex === false || $separatorIndex === strlen($currPageId) - 1) {
                    die('Illegal Marketing Automation Page ID returned from tracker: ' . $currPageId);
                }
                $currPage->channelId = substr($currPageId, 1, $separatorIndex - 1);
                $currPage->pageId = '10007.' . substr($currPageId, $separatorIndex + 1);
            } else {
                $currPage->channelId = NULL;
                $currPage->pageId = '10007.' . $currPageId;
            }
            $key = $currPage->pageId;
            if (!empty($currPage->channelId)) {
                $key = 'c' . $currPage->channelId . '.' . $key;
            }
            $ret[$key] = $currPage;
        }
        return $ret;
    }
    
    /**
     * Checks if any of the page IDs in the specified parsedPageIds array
     * already have page infos in the cache and if so, sets the cachedPageInfo
     * property of the corresponding parsedPageId to the cached page info.
     * 
     * @param array $parsedPageIds the array of parsed page IDs
     */
    private function _checkForCachedPageInfos(&$parsedPageIds) {
        foreach ($parsedPageIds as $pageId) {
            $cacheKey = $this->_getCacheKey($pageId->markaPageId);
            $cachedPageInfo = Yii::app()->cache->get($cacheKey);
            if ($cachedPageInfo !== false) {
                $pageId->cachedPageInfo = $cachedPageInfo;
            }
        }
    }
    
    /**
     * Groups the specified parsed page IDs, which do not have their cachedPageInfo
     * property set by their channel ID.
     * Returns an array, which contains an entry for each channelId. Each of 
     * these entries is an array of all the parsed page IDs, which belong
     * to that channel. Page IDs, which have NULL as their channel ID are 
     * placed in the entry with key 'master'. Page IDs, which have their
     * cachedPageInfo property set are not included.
     * 
     * @param array $parsedPageIds the array of parsed page IDs
     * @return array
     */
    private static function _groupPageIdsByChannel(&$parsedPageIds)
    {
        $ret = array();
        foreach ($parsedPageIds as $key => $pageId) {
            if (!empty($pageId->cachedPageInfo)) {
                continue;
            }
            $channelId = $pageId->channelId;
            if ($channelId === NULL) {
                $channelId = 'master';
            }
            if (!array_key_exists($channelId, $ret)) {
                $ret[$channelId] = array();
            }
            array_push($ret[$channelId], $pageId);
        }
        return $ret;
    }
    
    /**
     * Creates and returns a filter string with the specified parsed page IDs
     * for making a content repository request.
     * 
     * @param array $parsedPageIds the array of parsed page IDs
     * @return string
     */
    private static function _createFilterString(&$parsedPageIds)
    {
        $ret = 'object.contentid CONTAINSONEOF [';
        $first = true;
        foreach ($parsedPageIds as $pageId) {
            if (!$first) {
                $ret .= ',';
            } else {
                $first = false;
            }
            $ret .= '"' . $pageId->pageId . '"';
        }
        $ret .= ']';
        return $ret;
    }
    
    /**
     * Creates and returns an array, which contains all the page attributes that
     * should be retrieved from the content repository.
     * 
     * @return string
     */
    private function _createPageAttributesArray() {
        $baseAttributes = array('name', 'url');
        if (!empty($this->additionalPageAttributes)) {
            return array_merge($baseAttributes, $this->additionalPageAttributes);
        } else {
            return $baseAttributes;
        }
    }
    
    /**
     * Creates the final suggested pages infos array, which has the same order
     * as the page IDs, which were retrieved from the Marketing Automation system.
     * Pages, for which no page infos could be found, are not included in the
     * returned array.
     * 
     * @param array $parsedPageIds the array of parsed page IDs (the cachedPageInfo
     * property of each page ID is placed in the returned array)
     * @return array
     */
    private static function _createSuggestedPagesInfosArray(&$parsedPageIds) {
        $suggestedPagesInfos = array();
        $maxIndex = -1;
        foreach ($parsedPageIds as $pageId) {
            ++$maxIndex;
            if (!empty($pageId->cachedPageInfo)) {
                $suggestedPagesInfos[$pageId->posInSuggestions] = $pageId->cachedPageInfo;
            }
        }
        return self::_condenseArray($suggestedPagesInfos, $maxIndex);
    }
    
    /**
     * Creates and returns a non-sparse array, which contains all elements in the
     * specified array that have a numeric key (index), ordered by ascending
     * indices. E.g. input = array(0 => x1, 8 => x2, 2 => x3, 10 => x4), 
     * output = array(0 => x1, 1 => x3, 2 => x2, 3 => x4).
     * 
     * @param array $array the array that should be condensed
     * @param type $maxIndex the maximum index, up to which to iterate
     * @return array
     */
    private static function _condenseArray(&$array, $maxIndex) {
        $ret = array();
        for ($i = 0; $i <= $maxIndex; ++$i) {
            if (array_key_exists($i, $array)) {
                array_push($ret, $array[$i]);
            }
        }
        return $ret;
    }
    
    /**
     * Prepare API response. Convert response to useful form.
     * This method is based on RepositoryApi::_prepareResponse()
     *
     * @param array $response prepared response
     *
     * @return array|bool prepared object.
     *
     */
    private static function _prepareResponse($response)
    {
        $responseData = unserialize($response);
        if (@$responseData['status'] == 'ok') {
            unset($responseData['status']);
            if (count($responseData) > 0) {
                return $responseData;
            }
            return false;
        } else {
            Yii::log(print_r($responseData, true), CLogger::LEVEL_TRACE);
        }
        return false;
    }
    
    /**
     * Creates suggested page info associative arrays for all specified raw page infos
     * and inserts them into the cache and sets the cachedPageInfo property on
     * the corresponding parsed Page ID.
     * 
     * @param array $rawPageInfos the raw page infos
     * @param string $channelId the channel ID of all the pages in rawPageInfos
     * @param array $parsedPageIds the array of parsed page IDs, which should be used find the position of each page info in suggestedPageInfos
     * @param array $pageAttributes the page attributes that were supposed to be retrieved for each page
     */
    private function _createSuggestedPageInfos(&$rawPageInfos, &$channelId, &$parsedPageIds, &$pageAttributes)
    {
        if ($channelId !== 'master') {
            $channelPrefix = 'c' . $channelId . '.';
        } else {
            $channelPrefix = '';
        }
        foreach ($rawPageInfos as $pageId => $rawPageInfo) {
            $suggestedPageInfo = array();
            foreach ($pageAttributes as $attribute) {
                if (array_key_exists($attribute, $rawPageInfo['attributes'])) {
                    $suggestedPageInfo[$attribute] = $rawPageInfo['attributes'][$attribute];
                } else {
                    $suggestedPageInfo[$attribute] = NULL;
                }
            }
            if (empty($suggestedPageInfo)) {
                continue;
            }
            
            $parsedPageId = $parsedPageIds[$channelPrefix . $pageId];
            $parsedPageId->cachedPageInfo = $suggestedPageInfo;
            $cacheKey = $this->_getCacheKey($parsedPageId->markaPageId);
            if (Yii::app()->cache->set($cacheKey, $suggestedPageInfo, Yii::app()->getModule('marketingautomation')->cacheTime) === false) {
                Yii::log('MarkaSuggestionsWidget: Error adding page info to the cache.', CLogger::LEVEL_ERROR, 'php');
            }
        }
    }
    
    /**
     * Gets the cache key for the specified Marketing Automation Page ID.
     * 
     * @param string $markaPageId the Marketing Automation Page ID, for which to get the cache key
     * @return string
     */
    private function _getCacheKey($markaPageId)
    {
        return $this->cachePrefix . $markaPageId;
    }
}
